/*
 * @license
 * Your First PWA Codelab (https://g.co/codelabs/pwa)
 * Copyright 2019 Google Inc. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License
 */
'use strict';

// Update cache names any time any of the cached files change.
const CACHE_NAME = 'static-cache-v1';

// Add list of files to cache here.
const FILES_TO_CACHE = [
    '/',
    '/index.html',
    '/assets/vendors/custom/fullcalendar/fullcalendar.bundle.css',
    '/assets/demo/default/base/style.bundle.css',
    '/assets/my/plugins/bootstrap-table/bootstrap-table.min.css',
    '/assets/my/css/my-components.css',
    '/assets/my/plugins/bootstrap-fileinput/bootstrap-fileinput.css',
    '/assets/my/plugins/jquery-tags-input/jquery.tagsinput.css',
    '/assets/my/plugins/cropperjs/cropper.css',
    '/assets/my/plugins/bootstrap-tour/css/bootstrap-tour.min.css',
    '/assets/my/plugins/slider-revo/css/settings.css',
    '/assets/my/plugins/slider-revo/css/layers.css',
    '/assets/my/plugins/slider-revo/css/navigation.css',
    '/assets/my/plugins/amcharts/amcharts.js',
    '/assets/my/plugins/amcharts/serial.js',
    '/assets/my/plugins/amcharts/themes/light.js',
    '/assets/my/plugins/amcharts/pie.js',
    '/assets/my/plugins/amcharts/radar.js',
    '/assets/my/plugins/amcharts/plugins/animate/animate.min.js',
    '/assets/vendors/base/vendors.bundle.js',
    '/assets/demo/default/base/scripts.bundle.js',
    '/assets/my/plugins/bootstrap-tour/js/bootstrap-tour.min.js',
    '/assets/my/plugins/slider-revo/js/jquery.themepunch.tools.min.js',
    '/assets/my/plugins/slider-revo/js/jquery.themepunch.revolution.min.js',
    '/assets/my/plugins/slider-revo/js/extensions/revolution.extension.actions.min.js',
    '/assets/my/plugins/slider-revo/js/extensions/revolution.extension.carousel.min.js',
    '/assets/my/plugins/slider-revo/js/extensions/revolution.extension.kenburn.min.js',
    '/assets/my/plugins/slider-revo/js/extensions/revolution.extension.layeranimation.min.js',
    '/assets/my/plugins/slider-revo/js/extensions/revolution.extension.migration.min.js',
    '/assets/my/plugins/slider-revo/js/extensions/revolution.extension.navigation.min.js',
    '/assets/my/plugins/slider-revo/js/extensions/revolution.extension.parallax.min.js',
    '/assets/my/plugins/slider-revo/js/extensions/revolution.extension.slideanims.min.js',
    '/assets/my/plugins/slider-revo/js/extensions/revolution.extension.video.min.js',
];

self.addEventListener('install', (evt) => {
    console.log('[ServiceWorker] Install');
    // Precache static resources here.
    evt.waitUntil(
        caches.open(CACHE_NAME).then((cache) => {
            console.log('[ServiceWorker] Pre-caching offline page');
            return cache.addAll(FILES_TO_CACHE);
        })
    );
    self.skipWaiting();
});

self.addEventListener('activate', (evt) => {
    console.log('[ServiceWorker] Activate');
    // Remove previous cached data from disk.
    evt.waitUntil(
        caches.keys().then((keyList) => {
            return Promise.all(keyList.map((key) => {
                if (key !== CACHE_NAME) {
                    console.log('[ServiceWorker] Removing old cache', key);
                    return caches.delete(key);
                }
            }));
        })
    );

    self.clients.claim();
});

self.addEventListener('fetch', (evt) => {
    console.log('[ServiceWorker] Fetch', evt.request.url);
    // Add fetch event handler here.
    evt.respondWith(
        caches.open(CACHE_NAME).then((cache) => {
            return cache.match(evt.request)
                .then((response) => {
                    return response || fetch(evt.request);
                });
        })
    );

});
